# *****************************************************************
# *  Licensed Materials - Property of IBM
# *
# *  Copyright IBM Corp. 2010   All Rights Reserved.
# *
# *  US Government Users Restricted Rights - Use, duplication or 
# *  disclosure restricted by GSA ADP Schedule Contract with 
# *  IBM Corp.
# *
# *  This program is provided "AS IS" without warranty or indemnification 
# *  obligations of any kind. It is provided solely as a sample. It may be used,
# *  executed, copied, and modified for internal use only by Licensee 
# *  without royalty to IBM. 
# *
# *****************************************************************

"""
The functions in this module provide access to the clientAPI 
provided by the IBM WebSphere Appliance Management Toolkit.  
This is a string-driven interface (example, To display the attributes of 
a particular device call show_device("dp1") instead requiring the 
caller to keep track of a device object (awkward with command line 
and scripting). 

Sample Usage:
   from manager import *
   show_sets()
   create_device("dp123","dp123.mysomain.com", "myadmin", "passwd", 5550)
   load_firmware("file:///C:/Download/myDPfw.scrypt2")

Refer to the IBM WebSphere Appliance Management Toolkit JavaDoc for additional 
information about the clientAPI interface used by this module.
"""

from java.lang import * 
from java.util import * 
from java.util.logging import *
                                                                       
from com.ibm.datapower.wamt import * 
from com.ibm.datapower.wamt.amp import * 
from com.ibm.datapower.wamt.clientAPI import * 
from com.datapower.schemas.appliance.management.x20 import * 
from java.io import File

###########################################################
# INIT - Manager Commands
###########################################################

# Create the initial instance of the manager.  
# Use this method if options are needed for manager instantiation 
# example: To pass in a credential object, or designate an alternate 
# repository implementation.
# If no options are needed the get_manager() method can be used, but
# note that this method is called when this module is loaded.
def init_manager():
    options = HashMap();
#    options.put(Manager.OPTION_COMMANDS_IMPL, "com.ibm.datapower.wamt.amp.defaultProvider.CommandsImpl");
#    options.put(Manager.OPTION_NOTIFICATION_CATCHER_IMPL, "com.ibm.datapower.wamt.amp.defaultProvider.NotificationCatcherImpl");
#    options.put(Manager.OPTION_REPOSITORY_IMPL, "com.ibm.datapower.wamt.dataAPI.local.filesystem.RepositoryImpl");
    credential = Credential();
#    credential.setProperty("RepositoryEncryptionPassword", "your-repository-enc-pw");
    options.put(Manager.OPTION_CREDENTIAL, credential);
    manager = Manager.getInstance(options);
    return manager

#####################################################################

#  Use this method to get an existing instance of the manager. 
#  If no options are used to create the manager (see init_manager() 
#  for details) then this method can be used to get the first instance.
def get_manager():
    manager = Manager.getInstance(None);
    return manager

#####################################################################

# Use this method start an orderly shutdown of the manager. 
def shutdown():
    print "Shutdown manager"
    manager = get_manager();
    manager.shutdown();
    print "Shutdown completed"

#####################################################################

# Ask the manager to generate a report.
# The default path and file name are used.  
def generate_report():
    print "Generate Report"
    manager = get_manager();
    manager.generateReport(None, None);
    print "Report completed"
    
###########################################################
#
# "SHOW" COMMANDS                         SSSS
#                                        S
#                                         SSS
#                                            S
#                                        SSSS
#
###########################################################

# Show all of the Managed Sets defined in the Manager 
def show_sets():
    manager = get_manager();
    ms_array = manager.getManagedSets();
    if len(ms_array) == 0:
       print "No Managed Sets found"
    else:
       print  str(len(ms_array)) + " managed set(s) found"
       for i in range (0, len(ms_array)):
          print "";
          print "Managed Set: " + ms_array[i].getDisplayName();
          show_set(ms_array[i].getDisplayName());


#####################################################################

# Show the details of a Managed Set
#  params:
#   msName - the name of the Managed Set   
def show_set(msName):
    manager = get_manager();
    ms = manager.getManagedSet(msName);
    if ms is None:
       print "Managed Set " + msName + " not found"
    else: 
       dev_array = ms.getDeviceMembers();
       print "Found " + str(len(dev_array)) + " device(s)"
       for i in range (0, len(dev_array)):
          print "Device: " + dev_array[i].getSymbolicName();
          dom_array = dev_array[i].getManagedDomains() 
          print "Found " + str(len(dom_array)) + " managed domain(s) on this device"
          for j in range (0, len(dom_array)):
             print "  Managed Domain: " + dom_array[j].getName();

#####################################################################

#  Show all of the Devices defined in the Manager 
def show_devices():
    manager = get_manager();
    dev_array = manager.getAllDevices();
    if len(dev_array) == 0:
       print "No devices found"
    else:
       print  str(len(dev_array)) + " device(s) found"
       for i in range (0, len(dev_array)):
          print "Device: " + dev_array[i].getSymbolicName();

#####################################################################

#  Show the details of a Device 
#  params:
#   device - the symbolic name of the Device   
def show_device(device):
    manager = get_manager();
    device = manager.getDeviceBySymbolicName(device);
    if device is None:
       print "Device not found"
    else:
       print "Actual firmware level is " + device.getActualFirmwareLevel()
       print "Device Type:" + device.getDeviceType().getDisplayName()
       print "Model Type:" + device.getModelType().getDisplayName()
       print "Feature Licenses:" + device.getFeatureLicenses().toString()
       
       dom_array = device.getManagedDomains() 
       print "Found " + str(len(dom_array)) + " managed domain(s) on this device"
       for j in range (0, len(dom_array)):
          print "  Managed Domain:" + dom_array[j].getName() + " source:" + dom_array[j].getSourceConfiguration().getURL();

#####################################################################

# Show the Domain versions of a domain on a Device.
#  params:
#   device - the symbolic name of the Device containing the domain
#   domain - the name of the domain   
def show_domain_versions(device, domain):
    domainObj = find_domain(device, domain);
    if domainObj is not None:
        domver_array = domainObj.getVersions()
        print "Found " + str(len(domver_array)) + " domain version(s) for this domain"
        for i in range (0, len(domver_array)):
            print "Domain Version: " + domver_array[i].getAbsoluteDisplayName();

#####################################################################

# Show the Deployment Policy versions of a domain on a Device.
#  params:
#   device - the symbolic name of the Device containing the domain
#   domain - the name of the domain   
def show_policy_versions(device, domain):
    domainObj = find_domain(device, domain);
    if domainObj is not None:
        deppolver_array = domainObj.getDeploymentPolicy().getVersions()
        print "Found " + str(len(deppolver_array)) + " deployment policy version(s) for this domain"
        for i in range (0, len(deppolver_array)):
            print "Deployment Policy Version: " + deppolver_array[i].getAbsoluteDisplayName();

#####################################################################

# Show the Domain on a Device.
#  params:
#   device - the symbolic name of the Device containing the domain
#   domain - the name of the domain   
def show_domain(device, domain):
    manager = get_manager();
    device = manager.getDeviceBySymbolicName(device);
    domain = device.getManagedDomain(domain);
    print "Domain: " + domain.getAbsoluteDisplayName();

#####################################################################

# Show the Deployment Policy of a domain on a Device.
#  params:
#   device - the symbolic name of the Device containing the domain
#   domain - the name of the domain   
def show_policy(device, domain):
    manager = get_manager();
    device = manager.getDeviceBySymbolicName(device);
    domain = device.getManagedDomain(domain);
    deppol = domain.getDeploymentPolicy()
    print "Deployment Policy: " + deppol.getAbsoluteDisplayName();

#####################################################################

# Show all of the Firmware objects in the Manager
def show_firmwares():
    manager = get_manager();
    fw_array = manager.getFirmwares();
    if len(fw_array) == 0:
       print "No firmwares found in the manager"
    else:
       for i in range (0, len(fw_array)):
          print "Firmware: " + fw_array[i].getDisplayName();

#####################################################################

# Show all of the Firmware objects in the Manager that are compatible 
# with a specified device.
#  params:
#   device - the symbolic name of the device.
def show_firmwares_for_device(device):
    manager = get_manager();
    deviceObj=find_device(device);
    if deviceObj is not None:
        fw_array = manager.getFirmwares();
        if len(fw_array) == 0:
            print "No firmwares found in the manager"
        else:
            compatibleFirmwareCount = 0;
            for i in range (0, len(fw_array)):
                if fw_array[i].isCompatibleWith(deviceObj) == True:
                    compatibleFirmwareCount=compatibleFirmwareCount+1;
                    print "Firmware: " + fw_array[i].getDisplayName();
            if compatibleFirmwareCount == 0:
                print "No compatible firmwares found in the manager"

#####################################################################

# Show the Firmware object in the Manager that is most suitable for the 
# specified device.
#  params:
#   device - the symbolic name of the device.
#   level  - the desired firmware level - may be None
def show_best_firmware_for_device(device, level):
    best_fw = find_best_firmware(device, level);
    if best_fw is not None:
        print "Firmware: " + best_fw.getDisplayName();

###########################################################
#
# "FIND" COMMANDS
#
###########################################################

# Find the ManagedSet object that matches the name specified  
#  params:
#   setName - the name of the managed set
def find_set(setName):
    manager = get_manager();
    ms = manager.getManagedSet(setName);
    if ms is None:
       print "Managed Set " + setName + " not found"
#    else:
#       print "set:" + ms.getName();
    return ms
    
#####################################################################

# Find the Device object that matches the name specified
#  params:
#   deviceName - the symbolic name of the device
def find_device(deviceName):
    manager = get_manager();
    device = manager.getDeviceBySymbolicName(deviceName);
    if device is None:
       print "Device " + deviceName + " not found"
#    else:
#       print "device:" + device.getDisplayName();
    return device

#####################################################################

# Find the Domain object for a Device
#  params:
#   device - the symbolic name of the device
#   deviceName - the name of the domain
def find_domain(device, domainName):
    deviceObj = find_device(device);
    if deviceObj is not None:
        domainObj = deviceObj.getManagedDomain(domainName);
        if domainObj is None:
           print "Domain " + domainName + " not found"
#        else:
        return domainObj;

#####################################################################

# Find the Firmware object in the Manager that is most suitable for 
# the specified device.
#  params:
#   device - the symbolic name of the device.
#   level  - the desired firmware level - may be None
def find_best_firmware(device, level):
    manager = get_manager();
    deviceObj=find_device(device);
    if deviceObj is not None:
        best_fw = manager.getBestFirmware(deviceObj.getDeviceType(), deviceObj.getModelType(), deviceObj.getFeatureLicenses(), level);
        return best_fw

###########################################################
#
# DEVICE COMMANDS                        DDDD
#                                        D   D
#                                        D   D
#                                        D   D
#                                        DDDD
#
###########################################################

#  Create a new device in the manager
#  params:
#   symbolicName - a name that you want to associate with the new 
#                  device object. Used by show_device, etc. Typically 
#                  a short name to make scripting easier.
#   hostName     - the hostname or IP address of the device
#   id           - the administrative user ID for the device
#   pw           - password for th administrative user
#   port         - port # for the XML management interface
def create_device(symbolicName, hostName, id, pw, port):
    progressContainer = Device.createDevice(symbolicName, hostName, id, pw, port);
    progressContainer.blockAndTrace(Level.FINER);
    device = progressContainer.getResult();
    if progressContainer.getError() is None:
        return device
    else:
        print"  Error creating device:"+progressContainer.getError().getMessage() 

#####################################################################

#  Create a new managed domain on a Device 
#  params:
#   device     - the symbolic name of the device.
#   domainName - name of the new managed domain
def create_domain(device, domainName):
    deviceObj = find_device(device);
    if deviceObj is not None:
        domain = deviceObj.createManagedDomain(domainName);
        return domain

#####################################################################

#  Set the configuration source for a domain 
#  params:
#   device     - the symbolic name of the device.
#   domainName - name of the Managed Domain
#   url        - url to the source (example: file:///tmp/blah)
def set_domain_config(device, domainName, url):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
       urls = URLSource(url);
       domainObj.setSourceConfiguration(urls);
       print "Domain source for " + domainName + " on device " + device + " set to " + url

#####################################################################

#  Set the configuration source for a deployment policy 
#  params:
#   device       - the symbolic name of the device.
#   domainName   - the name of the Managed Domain
#   url          - url to the source (example: file:///tmp/blah)
#   policyDomain - the domain offset for the Deployment Policy 
#                      within the source 
#   policyName   - the Deployment Policy name within the source
def set_policy(device, domainName, url, policyDomain, policyName):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
       urls = URLSource(url);
       domainObj.setDeploymentPolicy(urls, policyDomain, policyName);
       print "Domain policy for " + domainName + " on device " + device + " set to " + url + " policy domain " + policyDomain + " policy name " + policyName

#####################################################################

#  Set the configuration source to "none" for a deployment policy 
#  params:
#   device       - the symbolic name of the device.
#   domainName   - the name of the Managed Domain
def set_dp_none(device, domainName):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
       domainObj.setDeploymentPolicy(None, None, None);
       print "Domain policy for " + domainName + " on device " + device + " set to NONE"

#####################################################################

#  Deploy a domain configuration to a device 
#  params:
#   device     - the symbolic name of the device.
#   domainName - the name of the Managed Domain
def deploy_domain(device, domainName):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
       progressContainer = domainObj.deployConfiguration();
       progressContainer.blockAndTrace(Level.FINER);
       if progressContainer.getError() is None:
           print " Deployed " + domainName + " on device " + device + " successfully"
       else:
           print " Error deploying " + domainName + " on device " + device

#####################################################################

#  Quiesce a device 
#  params:
#   device     - the symbolic name of the device.
def quiesce_device(device):
    deviceObj = find_device(device)
    if deviceObj is not None:
       deviceObj.quiesce();
    else:
        print " Can't find device:" + device

#####################################################################

#  Quiesce a domain on a device 
#  params:
#   device     - the symbolic name of the device.
#   domainName - the name of the Managed Domain
def quiesce_domain(device, domainName):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
       domainObj.quiesce(); 
    else:
       print " Error deploying " + domainName + " on device " + device

#####################################################################

#  Unquiesce a device 
#  params:
#   device     - the symbolic name of the device.
def unquiesce_device(device):
    deviceObj = find_device(device)
    if deviceObj is not None:
       deviceObj.unquiesce();
    else:
        print " Can't find device:" + device

#####################################################################

#  Unquiesce a domain on a device 
#  params:
#   device     - the symbolic name of the device.
#   domainName - the name of the Managed Domain
def unquiesce_domain(device, domainName):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
       domainObj.unquiesce(); 
    else:
       print " Can'd find domain " + domainName + " on device " + device

#####################################################################

#  Set the domain quiesce timeout for a domain 
#  params:
#   device     - the symbolic name of the device.
#   domainName - the name of the Managed Domain
#   timeout    - timeout value, must be greater than 60 seconds, or zero
def set_quiesce_timeout(device, domainName, timeout):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
       domainObj.setQuiesceTimeout(timeout);
       print "Quiesce timeout for " + domainName + " on device " + device + " set to " + str(timeout)

#####################################################################

#  Set the synchronization mode for a domain 
#  params:
#   device     - the symbolic name of the device.
#   domainName - the name of the Managed Domain
#   mode       - synchronization mode, "auto" or "manual"
def set_domain_synch_mode(device, domainName, mode):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
       if mode.lower() == "auto":
           domainObj.setSynchronizationMode(DomainSynchronizationMode.AUTO);
           print "Mode set to auto"
       elif mode.lower() == "manual":
           domainObj.setSynchronizationMode(DomainSynchronizationMode.MANUAL);
           print "Mode set to manual"
       else:
           print "Mode specified was " + mode + ". Valid values are 'auto' and 'manual'"

#####################################################################

#  Get the synchronization mode for a domain 
#  params:
#   device     - the symbolic name of the device.
#   domainName - the name of the Managed Domain
def get_domain_synch_mode(device, domainName):
    domainObj = find_domain(device, domainName)
    if domainObj is not None:
           mode = domainObj.getSynchronizationMode();
           print "Synchronization mode for " + domainName + " on " + device + " is " + mode.toString()

#####################################################################

#  Get the management status of the firmware on a device 
#  params:
#   device     - the symbolic name of the device.
def get_mgt_sts_of_firmware(device):
    deviceObj = find_device(device);
    if deviceObj is not None:
        ms = deviceObj.getManagementStatusOfFirmware();
        if ms is not None:
            print "Management status of the firmware on " + device + " is:" + ms.toString()
        else:
            print "Management status was not found"
#        return ms

#####################################################################

#  Get the operation status of a domain on a device 
#  params:
#   device - the symbolic name of the device.
#   domain - the name of the domain
def get_op_sts_of_domain(device, domain):
    deviceObj = find_device(device);
    domainObj = find_domain(device, domain);
    if (deviceObj is not None) and (domainObj is not None):
        os = deviceObj.getOperationStatusOfDomain(domainObj);
        if os is not None:
            print "Operation status of " + domain + " on " + device + " is:" + os.toString()
        else:
            print "Operation status was not found"
#        return os

#####################################################################

#  Get the rollup management status for the device 
#  params:
#   device     - the symbolic name of the device.
def get_rollup_mgt_sts(device):
    deviceObj = find_device(device);
    if deviceObj is not None:
        ms = deviceObj.getRollupManagementStatus();
        if ms is not None:
            print "Rollup management status for " + device + " is:" + ms.toString()
        else:
            print "Management status was not found"
#        return ms

#####################################################################

#  Get the rollup operationb status for the device 
#  params:
#   device     - the symbolic name of the device.
def get_rollup_op_sts(device):
    deviceObj = find_device(device);
    if deviceObj is not None:
        os = deviceObj.getRollupOperationStatus();
        if os is not None:
            print "Rollup operation status for " + device + " is:" + os.toString()
        else:
            print "Operation status was not found"
#        return os

#####################################################################

#  Remove a device from the manager. This also removes the device
#  from the managed set if it is a member of one. 
#  params:
#   device     - the symbolic name of the device.
def remove_device_from_manager(device):
    deviceObj = find_device(device);
    if deviceObj is not None:
       manager = get_manager();
       manager.remove(deviceObj);


###########################################################
#
# MANAGED SET COMMANDS                   M   M
#                                        MM MM
#                                        M M M
#                                        M   M
#                                        M   M
#
###########################################################

#  Create a new Managed Set in the Manager 
#  params:
#   set    - the name of the new Managed Set
def create_set(setname):
    set = ManagedSet(setname);
    return set

#####################################################################

# Add a Device to a Managed Set
#  params:
#   set    - the Managed Set that the device should be added to
#   device - the symbolic name of the device
def add_device(set, device):
    deviceObj = find_device(device);
    msObj = find_set(set);
    if deviceObj is None:
        print " Exiting because device was not found"
    else:
        if msObj is None:
            print " Exiting because managed set was not found"
        else:
            try:
               msObj.addDevice(deviceObj);
               print " Device " + deviceObj.getDisplayName() + " successfully added to set " + msObj.getName() + "."
            except:
               print " Error adding device " + deviceObj.getDisplayName() + " to set " + msObj.getName() + "."

#####################################################################

# Remove a Device from a Managed Set
#  params:
#   set    - the Managed Set that the device should be added to
#   device - the symbolic name of the device
def remove_device(set, device):
    deviceObj = find_device(device);
    msObj = find_set(set);
    if deviceObj is None:
        print " Exiting because device was not found"
    else:
        if msObj is None:
            print " Exiting because managed set was not found"
        else:
            msObj.removeDevice(deviceObj);
            print " Device " + deviceObj.getDisplayName() + " successfully removed from set " + msObj.getName() + "."

#####################################################################

# Create a Managed Domain object on every Device in a Managed Set
#  params:
#   set          - the Managed Set in which the Domain should be 
#                  created
#   domainName   - the name of the new Managed Domain
#   domainSource - the configuration source for the new Managed 
#                  Domain
#   policySource - the configuration source for the new Deployment 
#                  Policy
#   policyDomain - the domain offset for the Deployment Policy 
#                  within the source 
#   policyName   - the Deployment Policy name within the source
def create_domain_mset(set, domainName, domainSource, policySource, policyDomain, policyName):
    setObj = find_set(set);
    if setObj is not None:
        if domainSource is not None:
            domainUrlSource = URLSource(domainSource);
        else:
            domainUrlSource = None;
        
        if policySource is not None:
            policyUrlSource = URLSource(policySource);
        else:
            policyUrlSource = None;
        
        failedDevices = setObj.createManagedDomain(domainName, domainUrlSource, policyUrlSource, policyDomain, policyName);
        if failedDevices.isEmpty() == True:
            print " Managed domains created successfully for all devices in the managed set"
        else:
            print " Domains were not created successfully for these devices:" + failedDevices.toString()

#####################################################################

# Set the configuration source for domains in a managed set. This 
# operation is performed on the specified domain for each device 
# in the set.
#  params:
#   set          - the Managed Set that the device should be added to
#   domainName   - the name of the new Managed Domain
#   domainSource - the configuration source for the new Managed Domain
def set_domain_config_mset(set, domainName, domainSource):
    setObj = find_set(set);
    if setObj is not None:
        if domainSource is not None:
            domainUrlSource = URLSource(domainSource);
        else:
            domainUrlSource = None;
        
        failedDevices = setObj.setSourceConfigurationForDomain(domainName, domainUrlSource);
        if failedDevices.isEmpty() == True:
            print " Domain source set successfully for all devices in the managed set"
        else:
            print " Domain sorce was not created successfully for these devices:" + failedDevices.toString()

#####################################################################

#  Deploy a domain configuration to each device in a Managed Set 
#  params:
#   set        - the managed set name
#   domainName - the name of the Managed Domain
def deploy_domain_mset(set, domainName):
    setObj = find_set(set);
    if setObj is not None:
        macroProgressContainer = setObj.deploySourceConfigurationForDomain(domainName);
        macroProgressContainer.blockAndTrace(Level.FINER);
        if macroProgressContainer.hasError() is False:
            print "  Domain " + domainName + " deployed successfully to all devices in the set"
        else:
            print "  Failed to deploy domain on one or more devices."
            print "  Failure:" + macroProgressContainer.getError().getMessage() 

#####################################################################

#  Set the configuration source for a deployment policy on each 
#  Device in a Managed Set. 
#  params:
#   set          - the name of the Managed Set
#   domainName   - the name of the Managed Domain
#   policySource - url to the deployment policy source 
#                  (example: file:///tmp/blah)
#   policyDomain - the domain offset for the Deployment Policy 
#                      within the source 
#   policyName   - the Deployment Policy name within the source
def set_policy_mset(set, domainName, policySource, policyDomain, policyName):
    setObj = find_set(set);
    if setObj is not None:
        if policySource is not None:
            policyUrlSource = URLSource(policySource);
        else:
            policyUrlSource = None;
        
        failedDevices = setObj.setDeploymentPolicyForDomain(domainName, policyUrlSource, policyDomain, policyName);
        if failedDevices.isEmpty() == True:
            print " Deployment policy set successfully for all devices in the managed set"
        else:
            print " Set deployment policy failed for these devices:" + failedDevices.toString()

#####################################################################

# Set the quiesce timeout for the specified domain on each Device
# in a Managed Set.
#  params:
#   set        - the name of the Managed Set.
#   domainName - the name of the Managed Domain
#   timeout    - timeout value, must be greater than 60 seconds, or zero
def set_quiesce_timeout_mset(set, domainName, timeout):
    setObj = find_set(set);
    if setObj is not None:
        failedDevices = setObj.setQuiesceTimeoutForDomain(domainName, timeout);
        if failedDevices.isEmpty() == True:
            print " Quiesce timeout set successfully for all devices in the managed set"
        else:
            print " Set quiesce timeout failed for these devices:" + failedDevices.toString()

#####################################################################

# Set the synchronization mode for the specified domain on each Device 
# in a Managed Set 
#  params:
#   set        - the name of the Managed Set.
#   domainName - the name of the Managed Domain
#   mode       - synchronization mode, "auto" or "manual"
def set_synch_mode_mset(set, domainName, mode):
    setObj = find_set(set);
    if setObj is not None:
        if policySource is not None:
            policyUrlSource = URLSource(policySource);
        else:
            policyUrlSource = None;
        
        if mode.lower() == "auto":
            failedDevices = setObj.setQuiesceTimeoutForDomain(domainName, DomainSynchronizationMode.AUTO);
            print "Mode set to auto"
        elif mode.lower() == "manual":
           failedDevices = setObj.setQuiesceTimeoutForDomain(domainName, DomainSynchronizationMode.MANUAL);
           print "Mode set to manual"
        else:
           print "Mode specified was " + mode + ". Valid values are 'auto' and 'manual'"
           return
        
        failedDevices = setObj.setQuiesceTimeoutForDomain(domainName, timeout);
        if failedDevices.isEmpty() == True:
            print " Domain synchronization mode was set successfully set to " + mode + " for all devices in the managed set"
        else:
            print " Set domain synchronization mode failed for these devices:" + failedDevices.toString()

#####################################################################

# Remove the specified domain from each Device in a Managed Set.
#  params:
#   set        - the name of the Managed Set.
#   domainName - the name of the Managed Domain
def remove_domain_mset(set, domainName):
    setObj = find_set(set);
    if setObj is not None:
        failedDevices = setObj.removeManagedDomain(domainName);
        if failedDevices.isEmpty() == True:
            print " Managed domains successfully removed for all devices in the managed set"
        else:
            print " Domains were not successfully removed for these devices:" + failedDevices.toString()

#####################################################################

# Display the rollup operation status for each Device in a Managed Set.
#  params:
#   set        - the name of the Managed Set.
def get_operation_sts(set):
    setObj = find_set(set);
    if setObj is not None:
        os = setObj.getOperationStatus();
        if os is not None:
            print "The operation status of " + set + " is:" + os.toString();
        else:
            print "Operation status was not found"
#        return os

#####################################################################

# Remove the specified managed set from the Manager. The devices 
# in the managed set will still be defined in the Manager.
#  params:
#   set        - the name of the Managed Set.
def remove_set(set):
    setObj = find_set(set);
    if setObj is not None:
        manager = get_manager();
        manager.remove(setObj);
        print " Managed set " + set + " was successfully removed from the manager"

###########################################################
#
# FIRMWARE COMMANDS                      FFFFF
#                                        F
#                                        FFFFF
#                                        F
#                                        F
#
###########################################################

#####################################################################

# Load a firmware image into the manager 
#  params:
#   filename - the local file name of the firmware image
def load_firmware(filename):
    print "Loading firmware from " + filename
    file = File(filename);
    fwBlob = Blob(file);
    fwLevel = Utilities.getFirmwareLevel(fwBlob);
    manager = get_manager();
    progressContainer = manager.addFirmware(fwBlob, filename);
    progressContainer.blockAndTrace(Level.FINER);
    if progressContainer.getError() is None:
        print "  Firmware level " + fwLevel + " loaded OK"
    else:
        print "  Failed load firmware:"+progressContainer.getError().getMessage() 

#####################################################################

# Load a firmware via URL into the manager, source can be HTTP !!!
#  params:
#   url - a URL to the firmware image
def load_firmware_url(url):
    print "Loading firmware from " + url
    source = URLSource(url);
    fwBlob = source.getFirmwareBlob();
    fwLevel = Utilities.getFirmwareLevel(fwBlob);
    manager = get_manager();
    progressContainer = manager.addFirmware(fwBlob, url);
    progressContainer.blockAndTrace(Level.FINER);
    if progressContainer.getError() is None:
        print "  Firmware level " + fwLevel + " loaded OK"
    else:
        print "  Failed load firmware:"+progressContainer.getError().getMessage() 

#####################################################################

# Search the Manager for the best availabe firmware for a Device and 
# deploy it.
#  params:
#   device - device to update
#   level  - firmware level to deploy to the device
def deploy_best_firmware(device, level):
    manager = get_manager();
    device = manager.getDeviceBySymbolicName(device);
    if device is None:
       print "Device not found"
    else:
       try:
          device.setSourceFirmwareLevel(level)  # gets best firmware
          print "Deploying firmware, this will take a while..."
          progressContainer=device.deploySourceFirmwareVersion();
          progressContainer.blockAndTrace(Level.FINER);
          if progressContainer.getError() is None:
             print "  Firmware deploy OK"
          else:
             print "  Failed deploy:"+progressContainer.getError().getMessage()
       except:
          print "No suitable firmware found for " + dev_array[i].getSymbolicName();
                
#####################################################################

# Call deploy_best_firmware() for each device in the Managed Set
def deploy_best_firmware_mset(set, level):
    manager = get_manager();
    ms = manager.getManagedSet(set);
    if ms is None:
       print "Managed Set " + set + " not found"
    else: 
       dev_array = ms.getDeviceMembers();
       print "Found " + str(len(dev_array)) + " device(s)"
       for i in range (0, len(dev_array)):
          print "Searching for firmware level " +level+ "for device " + dev_array[i].getSymbolicName();
          deploy_best_firmware(dev_array[i].getSymbolicName(), level);
             
###########################################################
#
# Initial load                    III L
#                                  I  L
#                                 III LLL
#
###########################################################

init_manager()
print "Manager initialized"
print "---------------------------"   
print    
